"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseBridge = exports.CHAIN_IDS = exports.CHAIN_TO_ASSET_ID = void 0;
const fireblocks_sdk_1 = require("fireblocks-sdk");
const chain_1 = require("../interfaces/chain");
exports.CHAIN_TO_ASSET_ID = {
    [chain_1.Chain.MAINNET]: 'ETH',
    [chain_1.Chain.ROPSTEN]: 'ETH_TEST',
    [chain_1.Chain.GOERLI]: 'ETH_TEST3',
    [chain_1.Chain.KOVAN]: 'ETH_TEST2',
    [chain_1.Chain.BSC]: 'BNB_BSC',
    [chain_1.Chain.BSC_TEST]: 'BNB_TEST',
    [chain_1.Chain.POLYGON]: 'MATIC_POLYGON',
    [chain_1.Chain.MUMBAI]: 'MATIC_POLYGON_MUMBAI',
    [chain_1.Chain.RINKEBY]: 'ETH_TEST4',
    [chain_1.Chain.ARBITRUM]: 'ETH-AETH',
    [chain_1.Chain.FANTOM]: "FTM_FANTOM",
    [chain_1.Chain.AVALANCHE]: "AVAX",
};
exports.CHAIN_IDS = {
    [chain_1.Chain.MAINNET]: 1,
    [chain_1.Chain.ROPSTEN]: 3,
    [chain_1.Chain.RINKEBY]: 4,
    [chain_1.Chain.GOERLI]: 5,
    [chain_1.Chain.KOVAN]: 42,
    [chain_1.Chain.BSC]: 56,
    [chain_1.Chain.BSC_TEST]: 97,
    [chain_1.Chain.POLYGON]: 137,
    [chain_1.Chain.ARBITRUM]: 42161,
    [chain_1.Chain.MUMBAI]: 80001,
    [chain_1.Chain.FANTOM]: 250,
    [chain_1.Chain.AVALANCHE]: 43114,
};
class BaseBridge {
    constructor(params) {
        this.params = params;
        const chain = params.chain || chain_1.Chain.MAINNET;
        this.assetId = exports.CHAIN_TO_ASSET_ID[chain];
    }
    getDepositAddress() {
        return __awaiter(this, void 0, void 0, function* () {
            const depositAddresses = yield this.params.fireblocksApiClient.getDepositAddresses(this.params.vaultAccountId, this.assetId);
            return depositAddresses[0].address;
        });
    }
    getChainId() {
        return exports.CHAIN_IDS[this.params.chain];
    }
    waitForTxHash(txId, timeoutMs) {
        return __awaiter(this, void 0, void 0, function* () {
            return Promise.race([
                (() => __awaiter(this, void 0, void 0, function* () {
                    let status;
                    let txInfo;
                    while (!BaseBridge.finalTransactionStates.includes(status)) {
                        try {
                            txInfo = yield this.params.fireblocksApiClient.getTransactionById(txId);
                            status = txInfo.status;
                        }
                        catch (err) {
                            console.error(err);
                        }
                        if (txInfo.txHash) {
                            return txInfo.txHash;
                        }
                        yield new Promise(r => setTimeout(r, 1000));
                    }
                    if (status != fireblocks_sdk_1.TransactionStatus.COMPLETED) {
                        throw `Transaction was not completed successfully. Final Status: ${status}`;
                    }
                    return txInfo.txHash;
                }))(),
                new Promise((resolve, reject) => {
                    if (timeoutMs) {
                        setTimeout(() => reject(`waitForTxCompletion() for txId ${txId} timed out`), timeoutMs);
                    }
                })
            ]);
        });
    }
}
exports.BaseBridge = BaseBridge;
BaseBridge.finalTransactionStates = [
    fireblocks_sdk_1.TransactionStatus.COMPLETED,
    fireblocks_sdk_1.TransactionStatus.FAILED,
    fireblocks_sdk_1.TransactionStatus.CANCELLED,
    fireblocks_sdk_1.TransactionStatus.BLOCKED,
    fireblocks_sdk_1.TransactionStatus.REJECTED,
];
//# sourceMappingURL=base-bridge.js.map