"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Web3Bridge = void 0;
const fireblocks_sdk_1 = require("fireblocks-sdk");
const base_bridge_1 = require("./base-bridge");
const utils_1 = require("ethers/lib/utils");
const eth_sig_util_1 = require("eth-sig-util");
class Web3Bridge extends base_bridge_1.BaseBridge {
    signTypedData(typedData, txNote) {
        return __awaiter(this, void 0, void 0, function* () {
            const content = eth_sig_util_1.TypedDataUtils.sign(typedData).toString("hex");
            const transactionParams = {
                operation: fireblocks_sdk_1.TransactionOperation.RAW,
                assetId: this.assetId,
                source: {
                    type: fireblocks_sdk_1.PeerType.VAULT_ACCOUNT,
                    id: this.params.vaultAccountId
                },
                note: txNote || '',
                extraParameters: {
                    rawMessageData: {
                        messages: [{
                                content
                            }]
                    }
                }
            };
            const { id, status } = yield this.params.fireblocksApiClient.createTransaction(transactionParams);
            let txInfo;
            let currentStatus = status;
            while (!base_bridge_1.BaseBridge.finalTransactionStates.includes(currentStatus)) {
                try {
                    txInfo = yield this.params.fireblocksApiClient.getTransactionById(id);
                    currentStatus = txInfo.status;
                }
                catch (err) {
                    console.log("error:", err);
                }
                yield new Promise(r => setTimeout(r, 1000));
            }
            if (currentStatus != fireblocks_sdk_1.TransactionStatus.COMPLETED) {
                throw new Error(`Transaction was not completed successfully. Final Status: ${currentStatus}`);
            }
            const sig = txInfo.signedMessages[0].signature;
            const v = 27 + sig.v;
            return "0x" + sig.r + sig.s + v.toString(16);
        });
    }
    sendTransaction(transaction, txNote) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (transaction.chainId && transaction.chainId != this.getChainId()) {
                throw new Error(`Chain ID of the transaction (${transaction.chainId}) does not match the chain ID of the connected account (${this.getChainId()})`);
            }
            const txArguments = {
                operation: fireblocks_sdk_1.TransactionOperation.CONTRACT_CALL,
                assetId: this.assetId,
                source: {
                    type: fireblocks_sdk_1.PeerType.VAULT_ACCOUNT,
                    id: this.params.vaultAccountId
                },
                destination: {
                    type: this.params.externalWalletId ? fireblocks_sdk_1.PeerType.EXTERNAL_WALLET : fireblocks_sdk_1.PeerType.ONE_TIME_ADDRESS,
                    id: this.params.externalWalletId,
                },
                note: txNote || '',
                amount: (0, utils_1.formatEther)(((_a = transaction.value) === null || _a === void 0 ? void 0 : _a.toString()) || "0"),
                extraParameters: {
                    contractCallData: transaction.data
                }
            };
            if (transaction.to) {
                txArguments.destination.oneTimeAddress = {
                    address: transaction.to
                };
            }
            return this.params.fireblocksApiClient.createTransaction(txArguments);
        });
    }
}
exports.Web3Bridge = Web3Bridge;
//# sourceMappingURL=web3-bridge.js.map