"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ERC721 = void 0;
const base_token_1 = require("./base-token");
const base_abis_1 = require("../constants/base-abis");
const web3_1 = __importDefault(require("web3"));
class ERC721 extends base_token_1.BaseToken {
    constructor(bridgeParams) {
        super(bridgeParams, base_abis_1.ERC721_ABI);
    }
    /** Payable **/
    /**
     *
     * @param toAddress
     * @param tokenId
     * @param note
     */
    approve(toAddress, tokenId, note = "") {
        return __awaiter(this, void 0, void 0, function* () {
            const checkedToAddress = web3_1.default.utils.toChecksumAddress(toAddress);
            const transactionData = yield this.buildTransaction("approve", checkedToAddress, tokenId);
            return this.submitTransaction(transactionData, note);
        });
    }
    /**
     *
     * @param fromAddress
     * @param toAddress
     * @param tokenId
     * @param data
     * @param note
     */
    safeTransferFrom(toAddress, tokenId, data, note = "", fromAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            let transactionData;
            const checkedFromAddress = web3_1.default.utils.toChecksumAddress(fromAddress || (yield this.getAddress()));
            const checkedToAddress = web3_1.default.utils.toChecksumAddress(toAddress);
            if (data) {
                transactionData = yield this.contract.populateTransaction['safeTransferFrom(address,address,uint256,bytes)'](checkedFromAddress, checkedToAddress, tokenId, data);
            }
            else {
                transactionData = yield this.contract.populateTransaction["safeTransferFrom(address,address,uint256)"](checkedFromAddress, checkedToAddress, tokenId);
            }
            transactionData.from = checkedFromAddress;
            return this.submitTransaction(transactionData, note);
        });
    }
    /**
     *
     * @param fromAddress
     * @param toAddress
     * @param tokenId
     * @param note
     */
    transferFrom(toAddress, tokenId, note = "", fromAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            const checkedFromAddress = web3_1.default.utils.toChecksumAddress(fromAddress || (yield this.getAddress()));
            const checkedToAddress = web3_1.default.utils.toChecksumAddress(toAddress);
            const transactionData = yield this.buildTransaction("transferFrom", checkedFromAddress, checkedToAddress, tokenId);
            return this.submitTransaction(transactionData, note);
        });
    }
    /**
     *
     * @param operatorAddress
     * @param isApproved
     * @param note
     */
    setApprovalForAll(operatorAddress, isApproved, note = "") {
        return __awaiter(this, void 0, void 0, function* () {
            const checkedOperatorAddress = web3_1.default.utils.toChecksumAddress(operatorAddress);
            return this.submitTransaction(yield this.buildTransaction("setApprovalForAll", checkedOperatorAddress, isApproved), note);
        });
    }
    /** Views **/
    /**
     *
     * @param interfaceId
     */
    supportsInterface(interfaceId = '0x80ac58cd') {
        return this.callView("supportsInterface", interfaceId);
    }
    /**
     *
     * @param tokenId
     */
    getApproved(tokenId) {
        return this.callView("getApproved", tokenId);
    }
    /**
     *
     * @param ownerAddress
     * @param operatorAddress
     */
    isApprovedForAll(ownerAddress, operatorAddress) {
        const ownerCheckedAddress = web3_1.default.utils.toChecksumAddress(ownerAddress);
        const operatorCheckedAddress = web3_1.default.utils.toChecksumAddress(operatorAddress);
        return this.callView("isApprovedForAll", ownerCheckedAddress, operatorCheckedAddress);
    }
    /**
     *
     * @param ownerAddress
     */
    balanceOf(ownerAddress) {
        return __awaiter(this, void 0, void 0, function* () {
            const ownerCheckedAddress = web3_1.default.utils.toChecksumAddress(ownerAddress || (yield this.getAddress()));
            return this.callView("balanceOf", ownerCheckedAddress);
        });
    }
    /**
     *
     * @param tokenId
     */
    ownerOf(tokenId) {
        return this.callView("ownerOf", tokenId);
    }
}
exports.ERC721 = ERC721;
//# sourceMappingURL=erc721.js.map